from rest_framework import serializers
from clusterCore.models import c123_node, c123_host, c123_cluster, c123_inventory, Task, ClusterSizing, \
    IntegratedService
from clusterCore.enums import (
    DBType,
    ClusterRoles,
    Severity,
    Status,
    Inventory,
    TaskState,
    Schedule,
    DataBaseType,
    Node_Size_Allowed,
)
from clusterCore.api.serializer import (
    c123_CreateHostSerializer,
    c123_CreateTempClusterSerializer,
    object_to_json,
    RECREATION_CONFLICT, PRIMARY_GONE,
)
from clusterCore.exceptions import *
from clusterCore.tasks import (
    drop_backup,
    license_after_create,
    format_rsa_key,
    get_fully_qualified_domain_name,
    format_password, ensure_primary_created,
)
from apscheduler.triggers.interval import IntervalTrigger
from django.utils import timezone
import json


class c123_CreatePGClusterSerializer(serializers.HyperlinkedModelSerializer):
    cluster = c123_CreateTempClusterSerializer()

    class Meta:
        model = c123_node
        fields = ["cluster", "version", "database_user", "database_pwd", "sizing", "ex_params"]

    @license_after_create
    def create(self, validated_data):
        request_data = self.context["request"].data
        current_user = self.context["request"].user
        kubernetes_api_data = IntegratedService.objects.filter(pk=request_data["service_id"]).first()

        cluster_data = validated_data.pop("cluster")
        cluster_data.update({
            "created_by": current_user,
            "last_update_by": current_user,
            "type": DBType.POSTGRES_K8S.value,
        })
        cluster_data.setdefault("ex_params", {}).update({
            "instances": 1,
            "rw-service": f'{cluster_data["name"]}-rw',
            "ro-service": f'{cluster_data["name"]}-ro',
            "r-service": f'{cluster_data["name"]}-r',
        })
        cluster = c123_cluster.objects.create(**cluster_data)

        host_data = {
            "name": f"{cluster.name}-rw",
            "created_by": current_user,
            "last_update_by": current_user,
        }
        host_data.setdefault("ex_params", {}).update({
            "cluster_id": cluster.id,
        })
        host = c123_host.objects.create(**host_data)

        validated_data.update({
            "name": f"{cluster.name}-{cluster.ex_params['instances']}",
            "created_by": current_user,
            "last_update_by": current_user,
            "status": Status.CREATING.value,
            "severity": Severity.NOT_OK.value,
            "type": DBType.POSTGRES_K8S.value,
            "cluster_role": ClusterRoles.PRIMARY.value,
            "host": host,
            "cluster": cluster,
            "database_pwd": format_password(validated_data["database_pwd"]),
            "ind_deleted": False,
            "dir_123Cluster": "/var/lib/postgresql/data/pgdata",
        })
        validated_data.setdefault("ex_params", {})
        pod = c123_node.objects.create(**validated_data)

        pg_sys, created = ClusterSizing.objects.get_or_create(db_type=DBType.POSTGRES.value)
        match validated_data["sizing"]:
            case Node_Size_Allowed.SMALL.name:
                ram_gb = pg_sys.small_memory
                cpu_cores = pg_sys.small_cpu
                disk_gb = pg_sys.small_disk
            case Node_Size_Allowed.MEDIUM.name:
                ram_gb = pg_sys.medium_memory
                cpu_cores = pg_sys.medium_cpu
                disk_gb = pg_sys.medium_disk
            case Node_Size_Allowed.LARGE.name:
                ram_gb = pg_sys.large_memory
                cpu_cores = pg_sys.large_cpu
                disk_gb = pg_sys.large_disk
            case Node_Size_Allowed.VERY_LARGE.name:
                ram_gb = pg_sys.very_large_memory
                cpu_cores = pg_sys.very_large_cpu
                disk_gb = pg_sys.very_large_disk
            case _:
                ram_gb = 1
                cpu_cores = 1
                disk_gb = 10

        pod.ram_gb = f"{ram_gb}Gi"
        pod.cpu_cores = cpu_cores
        pod.disk_gb = f"{disk_gb}Gi"

        tmp_data = {
            "created_by": current_user.id,
            "kubernetes_api_data": object_to_json(kubernetes_api_data),
            "cluster_data": object_to_json(cluster),
            "pod_data": object_to_json(pod),
        }

        task = Task.objects.create(
            task="install_k8s_postgres_standalone_task",
            displayment="creating pg pod {}".format(pod.name),
            target_type=DBType.POSTGRES_K8S.value,
            target_id=pod.id,
            org_id=pod.org_id,
            status=TaskState.PENDING.value,
            schedule=Schedule.NOW.value,
            parameters=json.dumps(tmp_data),
            created_by=current_user,
        )

        return pod, task


class c123_AddPGStandbySerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = c123_node
        fields = ["ex_params"]

    @license_after_create
    def create(self, validated_data):
        request_data = self.context["request"].data
        current_user = self.context["request"].user
        kubernetes_api_data = IntegratedService.objects.filter(pk=request_data["service_id"]).first()

        cluster = c123_cluster.objects.filter(pk=request_data["cluster_id"]).first()
        cluster.last_update_by = current_user
        cluster.ex_params["instances"] += 1
        cluster.save()

        host_data = {
            "name": f"{cluster.name}-ro",
            "created_by": current_user,
            "last_update_by": current_user,
        }
        host_data.setdefault("ex_params", {}).update({
            "cluster_id": cluster.id,
        })
        host, created = c123_host.objects.update_or_create(name=host_data["name"], ex_params__cluster_id=cluster.id, ind_deleted=False, defaults=host_data)

        primary = c123_node.objects.filter(cluster=cluster, cluster_role=ClusterRoles.PRIMARY.value, ind_deleted=False).first()

        validated_data.update(
            {
                "name": f"{cluster.name}-{cluster.ex_params['instances']}",
                "created_by": current_user,
                "last_update_by": current_user,
                "status": Status.CREATING.value,
                "severity": Severity.NOT_OK.value,
                "type": DBType.POSTGRES_K8S.value,
                "cluster_role": ClusterRoles.STANDBY.value,
                "host": host,
                "cluster": cluster,
                "version": primary.version,
                "database_user": primary.database_user,
                "database_pwd": format_password(primary.database_pwd),
                "ind_deleted": False,
                "dir_123Cluster": primary.dir_123Cluster,
                "sizing": primary.sizing,
            }
        )
        validated_data.setdefault("ex_params", {}).update(primary.ex_params)
        pod, created = c123_node.objects.update_or_create(name=validated_data["name"], cluster=cluster.id, defaults=validated_data)

        tmp_data = {
            "created_by": current_user.id,
            "kubernetes_api_data": object_to_json(kubernetes_api_data),
            "cluster_data": object_to_json(cluster),
            "pod_data": object_to_json(pod),
        }

        task = Task.objects.create(
            task="add_k8s_standby_task",
            displayment="adding pg standby {}".format(pod.name),
            target_type=DBType.POSTGRES_K8S.value,
            target_id=pod.id,
            org_id=pod.org_id,
            status=TaskState.PENDING.value,
            schedule=Schedule.NOW.value,
            parameters=json.dumps(tmp_data),
            created_by=current_user,
        )

        return pod, task
